@@ -1,14 +1,16 @@
 import yaml as _yaml
 
-from bottles.backend.models.config import BottleConfig
-
 try:
     from yaml import CSafeLoader as SafeLoader, CSafeDumper as SafeDumper
 except ImportError:
     from yaml import SafeLoader, SafeDumper
 
 YAMLError = _yaml.YAMLError
-SafeDumper.add_representer(BottleConfig, BottleConfig.yaml_serialize_handler)
+
+
+def register_serializer(cls):
+    """Register a custom serializer for SafeDumper."""
+    SafeDumper.add_representer(cls, cls.yaml_serialize_handler)
 
 
 def dump(data, stream=None, **kwargs):
